# for creating binary package

pack_name()
{
    if [ "${BP}" != "${P}" ] ; then
        for entry in ${BP} ; do
            printf "%s\n" "${entry}"
        done
    else
        for entry in ${BPN} ; do
            printf "%s\n" "${entry}-${BPV}"
        done
    fi
}

pack_name_base()
{
    if [ "${BP}" != "${P}" ] ; then
        for entry in ${BP} ; do
            printf "%s\n" "$(depend_get_pack_name ${entry})"
        done
    else
        for entry in ${BPN} ; do
            printf "%s\n" "${entry}"
        done
    fi
}

pack_name_from_shortname()
{
    base="${1}"

    versname=
    if test "${BP}" != "${P}" ; then
        for word in ${BP} ; do
            case ${word} in
                ${base}-[0-9]*)
                    versname="${word}"
                    break
                ;;
            esac
        done
    else
        versname="${base}-${BPV}"
    fi
    if test -n "${versname}" ; then
        printf "%s\n" "${versname}"
    fi
}

pack_name_file_actual()
{
    keywords_check
    status_bool || return
    pack_name_file
}

pack_fallback_allowed()
{
    # if current package produces single binary archive, fallback materials can be used for it:
    # ${D} instead of ${IMGDID}/${packname} for binaries,
    # postinst script name instead of ${packname}.postinst and so on

    fallback_allowed=
    if test "${BP}" != ${P} ; then
        if test "${BP#* }" = "${BP}" ; then
            fallback_allowed=1
        fi
    elif test "${BPN#* }" = "${BPN}" ; then
        fallback_allowed=1
    fi

    if test -n "${fallback_allowed}" ; then
        printf "%s\n" "${fallback_allowed}"
    fi
}

pack_dir_for_runtime_entry()
{
    pack="$1"
    fallback="$2"

    dir=
    if test -d "${IMGDIR}/${pack}" ; then
        dir="${IMGDIR}/${pack}"
    elif test -n "${fallback}" -a -d "${D}" ; then
        dir="${D}"
    else
        # check the name without version, when version could be autogenerated
        barename_pack="${pack%-0.0}"
        if test "${barename_pack}" != "${pack}" ; then
            if test -d "${IMGDIR}/${barename_pack}" ; then
                dir="${IMGDIR}/${barename_pack}"
            fi
        fi
    fi
    if test -n "${dir}" ; then
        printf "%s\n" "${dir}"
    fi
}

pack_metafile_for_runtime_entry()
{
    pack="$1"
    script="$2"
    fallback="$3"

    filename=
    if test -f "${DISTDIR}/${pack}.${script}" ; then
        filename="${DISTDIR}/${pack}.${script}"
    elif test -n "${fallback}" ; then
        basedir_for_script="${pack%/*}"
        if test "${basedir_for_script}" = "${pack}" ; then
            basedir_for_script=
        fi
        if test -f "${DISTDIR}/${basedir_for_script}/${script}" ; then
            filename="${DISTDIR}/${basedir_for_script}/${script}"
        fi
    fi
    if test -n "${filename}" ; then
        printf "%s\n" "${filename}"
    fi
}

# remove given temporary files or directories in all subpackages

pack_clean_tmp_subpackage()
{
    cleanfiles="$1"
    fallback="$2"

    if test -z "${cleanfiles}" ; then
        return 1
    fi
    if test -z "${fallback}" ; then
        fallback="$(pack_fallback_allowed)"
    fi

    for binname in $(pack_name) ; do
        bindir="$(pack_dir_for_runtime_entry "${binname}" "${fallback}" )"
        for object in ${cleanfiles} ; do
            rm -rf "${bindir}/${object}"
        done
    done
}

# read the format
# :subpackage_one:
# string1
# string2
# :subpackage_two:
# one
# two
# three

pack_parse_subpackage_text()
{
    basefile="$1"
    fallback="$2"

    if test -z "${basefile}" ; then
        return 1
    fi
    if test -z "${fallback}" ; then
        fallback="$(pack_fallback_allowed)"
    fi
    subdir="${basefile%/*}"
    if test "${subdir}" = "${basefile}" ; then
        subdir=
    fi
    # clumsy, but need to manage output of 'while' somehow
    mkdir -p ${T}/packparse/${subdir}

    subname=

    while IFS='' read line ; do
        case "${line}" in
            # :token:
            :[^[:space:]:]*:)

                # read the next subname in current string
                key="${line#*:}"
                subname="${key%:*}"

                # prepare the next subpackage
                binname="$( pack_name_from_shortname "$subname" )"
                bindir="$( pack_dir_for_runtime_entry "$binname" "${fallback}" )"
                if test -n "${bindir}" ; then
                    mkdir -p "${bindir}/${subdir}"
                else
                    awarn "No directory for subpackage ${binname} (of $P) was found. Data is not saved to ${basefile}."
                fi
            ;;

            *)
                # dusky twist as inner variables will perish after the current cycle
                # so unhandled text is printed to be saved
                if test -z "${subname}" ; then
                    # no subpackage, handle such text in outer code
                    printf "%s\n" "${line}"
                else
                    # normal text handling
                    if test -n "${bindir}" ; then
                        printf "%s\n" "${line}" >> ${bindir}/${basefile}
                    fi
                fi
            ;;
        esac
    done <& 0 > ${T}/packparse/${basefile}

    if test -s "${T}/packparse/${basefile}" ; then
        mkdir -p ${D}/${subdir}
        cat ${T}/packparse/${basefile} >> ${D}/${basefile}
    else
        rm ${T}/packparse/${basefile}
    fi

}

pack_create()
{
    archives="$(pack_name_file)"
    packs="$(pack_name)"

    fallback="$(pack_fallback_allowed)"

    mkdir -p ${PACKDIR}
    for entry in ${packs} ; do
        fileentry="$(get_first_string "${archives}")"
        archives="$(delete_first_string "${archives}" "${fileentry}" )"
        if test -z "${fileentry}" ; then
            die "No file for bin package ${entry}"
        fi

        installed_dir="$(pack_dir_for_runtime_entry ${entry} ${fallback})"
        if test -z "${installed_dir}" ; then
            die "No directory to create package from ${entry}"
        fi

        pack_create_archive ${installed_dir} ${PACKDIR}/${fileentry}

    done
}

pack_dump_content()
{
    path_to_dump="$1"
    if test -z "${path_to_dump}" ; then
        path_to_dump="${T}/dumpcontent"
    fi
    mkdir -p ${path_to_dump}

    params=

    if test ! -t 0 ; then
        while read line_input_pack <&0 ; do
            for input_pack in ${line_input_pack} ; do
                params=1
                cur_package="${input_pack}"
                if test ! -f "${cur_package}" ; then
                    cur_package=${PACKDIR}/${cur_package}
                fi
                if test ! -f "${cur_package}" ; then
                    awarn "Package ${input_pack} was not found."
                    continue
                fi
                pack_dump_content_archive ${cur_package} ${path_to_dump}
            done
        done
    fi

    if test -z "${params}" ; then
        pack_name_file | while read input_pack ; do
            cur_package="${input_pack}"
            if test ! -f "${cur_package}" ; then
                cur_package="${PACKDIR}/${cur_package}"
            fi
            if test ! -f "${cur_package}" ; then
                awarn "Package ${input_pack} was not found."
                continue
            fi
            pack_dump_content_archive ${cur_package} ${path_to_dump}
        done
    fi
}

# print part of the text, concerning the subpackage ${subname}
pack_extract_subname_info()
{
    text="${1}"
    subname="${2}"
    fallback="${3}"

    if test -n "${fallback}" ; then
        printf "%s\n" "${text}"
        return
    fi

    temp_text="${text##*:${subname}:}"
    result="${temp_text%%:[^[:space:]]*:*}"
    if test "${result}" != "${text}" ; then
        printf "%s\n" "${result}"
    fi
}

# -------------------------------------------------------------------
# implementation specific interfaces

pack_name_file()
{
    need_real_files="$1"
    if test -n "${need_real_files}" ; then
        if test -n "${any_pkg}" ; then
            pack_dir="$(cd ${any_pkg} ; pwd -P)"
        else
            pack_dir="${PACKDIR}"
        fi
        # assumption: do not use strict inner versions for real files.
        # instead check matching files with possible various inner versions.
        # if one needs real files + strict inner versions, he should call pack_name_file without the argument
        # and check the existence of files outside of the call
        # (since the file name is single entry and not a template of possible ones, that is straighforward)

        inner_version=
    else
        inner_version="$(fetch_inner_version)"
    fi

    arch_default="${ANYARCH}"
    fallback="$(pack_fallback_allowed)"

    explicit_arch=
    if test -n "${PKG_ARCHITECTURE}" ; then
        explicit_arch="${PKG_ARCHITECTURE}"
    fi
    if test -n "${explicit_arch}" ; then
        meta_arch="TMPNAME/arch"
        pack_clean_tmp_subpackage "${meta_arch}"
        printf "%s\n" "${explicit_arch}" | pack_parse_subpackage_text "${meta_arch}" "${fallback}"
    fi

    flav_string=
    for fl_word in ${FLAVOUR} ; do
        flav_string="${flav_string}-${fl_word}"
    done
    if [ "${BP}" != "${P}" ] ; then
        entry_array="${BP}"
        flag_version_combined=1
    else
        entry_array="${BPN}"
        flag_version_combined=
    fi
    for entry in ${entry_array} ; do
        if test -n "${flag_version_combined}" ; then
            binname="${entry}"
            pack_try_name="$(depend_get_pack_name ${entry})"
        else
            binname="$(pack_name_from_shortname ${entry})"
            pack_try_name="${entry}"
        fi
        bindir=$(pack_dir_for_runtime_entry "${binname}" "${fallback}" )
        arch=

        if test -n "${explicit_arch}" ; then
            arch="$( cat "${bindir}/${meta_arch}" )"
            rm -rf "${bindir}/TMPNAME/"
        fi
        if test -z "${arch}" ; then
            arch="${arch_default}"
        fi
        if test -n "${ANYOS}" ; then
            arch="${arch}-${ANYOS}"
        fi

        if test -n "${flag_version_combined}" ; then
            pack_try_version="$(depend_get_pack_version ${entry})"
        else
            pack_try_version="${BPV}"
        fi
        pack_try_name="${pack_try_name}${flav_string}"

        if test -n "${need_real_files}" ; then
            # skip inner version on purpose - see comment at the beginning
            # expanding several files by * symbol without outer binary
            expansion="$( echo "${pack_dir}/${pack_try_name}-${pack_try_version}-${arch}"*".tgz" )"

            # the problem here is the last position in string would be alhabetical: the maximum digit
            # but it should be the maximum version (11 versus 7, for example)
            # so need to find maximum explicitly
            pack_try=
            for candidate in ${expansion} ; do
                if arith_greater_then "${pack_try}" "${candidate}" ; then
                    pack_try="${candidate}"
                fi
            done
            if test -s "${pack_try}" ; then
                printf "%s\n" "${pack_try}"
            fi
        else
            pack_try="${pack_try_name}-${pack_try_version}-${arch}${inner_version}.tgz"
            printf "%s\n" "${pack_try}"
        fi
    done
}

pack_create_archive()
{
    sourcedir="$1"
    archivefile="$2"

    alog "Creating binary package: plain tar archive ${archivefile}"
    cd ${sourcedir}
    ${UTILROOT} tar czvf ${archivefile} *
}

pack_gen_meta()
{
    archives="$(pack_name)"
    fallback="$(pack_fallback_allowed)"

    for pack in ${archives} ; do
        dir="$(pack_dir_for_runtime_entry ${pack} ${fallback})"
        if test -z "${dir}" ; then
            continue
        fi

        (
        cd ${dir}

        mkdir -p var/lib/pkg/files/
        find * > var/lib/pkg/files/${pack}

        basepack="$(depend_get_pack_name ${pack})"
        postinst_file="$(pack_metafile_for_runtime_entry ${basepack} 'postinst' ${fallback} )"
        if test -n "${postinst_file}" ; then
            mkdir -p var/lib/pkg/scripts/
            cp -f ${postinst_file} var/lib/pkg/scripts/${pack}
            chmod 555 var/lib/pkg/scripts/${pack}
        fi
        triggers_file="$(pack_metafile_for_runtime_entry ${basepack} 'triggers' ${fallback} )"
        if test -n "${triggers_file}" ; then
            mkdir -p var/lib/pkg/triggers/
            cp -f ${triggers_file} var/lib/pkg/triggers/${pack}
            chmod 555 var/lib/pkg/triggers/${pack}
        fi
        )
    done
}

# save info about binary packages (including file listing) and print it
pack_info()
{
    return
}

# print full version of given binary package
pack_get_version()
{
    return
}

# print inner part of version of given binary package
# that does not include such parts of version, as ${RELEASE} string
# only number
pack_get_inner_version()
{
    return
}

# read name of the file and print the package, it belongs to
pack_define_by_filename()
{
    return
}

# meta-package creator:
# print given package set, check the existense of real files
# and convert package names to dependency format
pack_sum()
{
    return
}

# deploy single binary package to IMGDIR
pack_deploy_single_archive()
{
    single_archive="$1"
    fallback="$2"

    if test ! -f "${single_archive}" ; then
        awarn "no expected pack ${single_archive}"
        return 4
    fi
    alog "Deploy archive ${single_archive}"

    if test -n "${fallback}" ; then
        destdir="${D}"
    else
        # deploy each archive into separate dir, need to calculate dir name
        template_name="${single_archive##*/}"
        # remove file extension
        template_name="${template_name%.txz}"
        # remove operating system
        template_name="${template_name%-*}"
        # remove arch
        pack_full="${template_name%-*}"

        destdir="${IMGDIR}/${pack_full}"
    fi

    mkdir -p "${destdir}"
    tar xf "${single_archive}" -C "${destdir}"
}

# deploy binary packages to IMGDIR
pack_unpack()
{
    rm -rf "${D}"
    fallback="$(pack_fallback_allowed)"
    binpkg_names="$(pkgbinfiles)"

# sorrowfully, we cannot use direct pipe and do intermediate variable, which is slower.
# If piped pkgbinfiles fails, it is not detected anywhere, execution continues.
# Then method happily returns 0 status, while real state is failed.
#    pkgbinfiles | while read pack ; do

    printf "%s\n" "${binpkg_names}" | while read pack ; do
        pack_deploy_single_archive "${pack}" "${fallback}" || \
            die "Expected ${pack} had not been found"
    done
}

# save file listing of binary packages in files
pack_dump_content_archive()
{
    archive="$1"
    dumppath="$2"

    archivename="${archive##*/}"
    tar tzf "${archive}" > "${dumppath}/${archivename}.txt"
}
